==> release.12.2.28.1.tgz: cgi-cpn/prontus_purge_cache.pl <==
#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
use strict;
use utf8;


BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}


END {
    &delete_semaphore();
}

$SIG{INT} = \&signal_callback;
$SIG{TERM} = \&signal_callback;

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use HTTP::Request;
use JSON;
use LWP::ConnCache;
use LWP::UserAgent;
use Time::HiRes;
use glib_dbi_03;
use glib_fildir_03;
use lib_bd;
use lib_urlf5;
use lib_utils;
##/libs##

my $UA;
my $DIR_SEMAF;
my %PURGED_FILES;
my $FILE_LIFE_SEGS = 5;
my $RUNNING = 0;
my @SUBDOMAINS;

main: {
    lib_utils::setUtf8();
    $prontus_varglb::PRONTUS_ID = $ARGV[0];

    if (scalar @ARGV < 1) {
        print "Parametros insuficientes Terminando\n";
        exit;
    }

    if ($prontus_varglb::PRONTUS_ID eq '' || !lib_utils::valida_prontus($prontus_varglb::PRONTUS_ID)) {
        print "Error: El identificador de Prontus es inválido [$prontus_varglb::PRONTUS_ID]\n";
        print STDERR "Error: El identificador de Prontus [$prontus_varglb::PRONTUS_ID] es inválido\n";
        exit;
    }

    # Carga variables de configuracion de prontus.
    my $msg_load = lib_utils::load_config_by_id($prontus_varglb::PRONTUS_ID);
    if ($msg_load ne '') {
        print STDERR "Error: $msg_load\n";
        exit;
    }

    # Conectar a BD
    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        print STDERR "ERROR: $msg_err_bd\n";
        exit;
    }

    @SUBDOMAINS = keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_SUBDOMAINS'}};

    # agrega elemento vacío para procesar el dominio principal
    push (@SUBDOMAINS, '');

    $DIR_SEMAF = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/semaforos";
    glib_fildir_03::check_dir($DIR_SEMAF);

    if (-f "$DIR_SEMAF/purge_cache.lck") {
        my $mtime = (stat("$DIR_SEMAF/purge_cache.lck"))[9];
        my $now = time;
        my $diff = $now - $mtime;

        if ($diff > 1800) { # 30 min.
            print STDERR "[$$] Semaforo muy antiguo, eliminando...";
            unlink "$DIR_SEMAF/purge_cache.lck";

            my $res = `ps auxww |grep 'prontus_purge_cache.pl $prontus_varglb::PRONTUS_ID' | grep -v grep | wc -l`;
            chomp($res);

            if ($res) {
                system('kill -9 `ps auxww | grep \'prontus_purge_cache.pl ' . $prontus_varglb::PRONTUS_ID . '\' | grep -v grep | awk \'{print $2}\' | grep -v ' . $$ . '`');
            }
        } else {
            exit(0); # ya existe un proceso.
        }
    }

    # escribe el semaforo y ya esta corriendo.
    glib_fildir_03::write_file("$DIR_SEMAF/purge_cache.lck", $$);
    $RUNNING = 1;

    my $espera_segs = 5;
    $UA = new LWP::UserAgent;
    $UA->timeout($espera_segs); # segs # default es 180
    $UA->agent('Mozilla/5.0 (Windows; U; Windows NT 5.1; es-ES; rv:1.9.0.3) Gecko/2008092417 Firefox/3.0.3');
    # no verifica certificados para poder acceder a sitios por ip que pueden entregar un certificado invalido
    $UA->ssl_opts(verify_hostname => 0,
              SSL_verify_mode => 0x00);
    my $conn_cache = LWP::ConnCache->new();
    $UA->conn_cache($conn_cache);

    my $dir_purgepend = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/purgepend";
    my @purgepend_chunklist = &get_purgepend_list($dir_purgepend);
    my $chunk_ref;

    while (defined($chunk_ref = shift @purgepend_chunklist)) {
        my @chunk = @{$chunk_ref};
        my %filelist;

        foreach my $file (@chunk) {
            if (-f $file) {
                print STDERR "[$$] Procesando cola [$file]...\n";
                %filelist = &get_filelist_hash($file, $bd, \%filelist);
            }
        }

        my $num_of_files = keys %filelist;

        print STDERR "[$$] Rutas a procesar: $num_of_files\n";

        if ($num_of_files) {
            &purge(\%filelist);
        }

        # Borrar archivos
        foreach my $file (@chunk) {
            unlink $file if (-f $file);
        }

        if (!scalar @purgepend_chunklist) {
            print STDERR "[$$] Buscando mas archivos en la cola...\n";
            @purgepend_chunklist = &get_purgepend_list($dir_purgepend);
        }
    }

    print STDERR "[$$] Fin.\n";

    unlink "$DIR_SEMAF/purge_cache.lck";
}


sub get_purgepend_list {
    my $dir_purgepend = $_[0];

    my @files = glob("$dir_purgepend/*.txt");
    my @chunks;

    # Dividir array de archivos.
    push @chunks, [splice @files, 0, 50] while (@files); # grupos de 50 archivos.

    return @chunks;
}


sub get_filelist_hash {
    my $file = $_[0];
    my $bd = $_[1];
    my %hash = %{$_[2]};

    open(my $PURGEFILE, '<', "$file");

    foreach my $filetopurge (<$PURGEFILE>) {
        $filetopurge =~ s/\n//s;
        my $relpath = lib_utils::remove_front_string($filetopurge, $prontus_varglb::DIR_SERVER);
        if (!exists $hash{$relpath}) {
            $hash{$relpath} = time;
            # friendly v5 activa
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
                next if ($relpath !~ /\.(html|htm|php|asp)$/);
                my $result = lib_urlf5::get_file_data($bd, $relpath);
                if ($result) {
                    $hash{$result->{'URLF5_URL'}} = time;
                }
            }
        }
    }
    close $PURGEFILE;

    return %hash;
}


sub purge {
    my %hash = %{$_[0]};

    print STDERR "[$$] Servidores de cache activos.\n";

    # Agregar purge global al listado.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_GLOBAL_PURGE'} ne '') {
        my @arr = split(/[\n\r]/, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_GLOBAL_PURGE'});
        foreach my $path (@arr) {
            next unless($path);
            next if ($path !~ /^\//); # debe empezar con /

            $hash{$path} = time if (!exists $hash{$path});
        }
    }

    # purga las rutas desde los servidores de cache
    foreach my $relpath (keys %hash) {
        if (defined $PURGED_FILES{$relpath} && &check_life($PURGED_FILES{$relpath})) {
            print STDERR "[$$] Saltando [$relpath].\n";
            next;
        }

        $PURGED_FILES{$relpath} = time;

        foreach my $server (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}}) {
            next if (!$server);
            my $url_purge;
            if ($server =~ /^https?:\/\//) {
                $url_purge = "$server$relpath";
            } else {
                $url_purge = "https://$server$relpath";
            }

            my ($resp, $err) = &http_purge($url_purge);
            print STDERR "[$$] cache: server[$server], url_purge[$url_purge], status[$err]\n";
        }
    }

    if (   $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE'} eq 'SI'
        && $prontus_varglb::CLOUDFLARE_API_URL ne ''
        && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_API_TOKEN'} ne ''
        && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_ZONE'} ne ''){
        # asumimos que la zona de CLOUDFLARE_ZONES es el id alfanumerico de zona, no el dominio. Como dominio se usa el PUBLIC_SERVER_NAME.
        my $cloudflareZone = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_ZONE'};
        my $cloudflareDomain = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'};

        print STDERR "[$$] Cloudflare activado. CLOUDFLARE_API_URL[$prontus_varglb::CLOUDFLARE_API_URL]\n";

        # Agregar purge global al listado.
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_GLOBAL_PURGE'} ne '') {
            my @arr = split(/[\n\r]/, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_GLOBAL_PURGE'});

            foreach my $path (@arr) {
                next unless($path);
                next if (substr($path, 0, 1) ne '/'); # debe empezar con /

                $hash{$path} = time if (!exists $hash{$path});
            }
        }

        # Se envian a la api grupos de 30 url.
        my @archivos = (keys %hash);
        my @chunks;
        push @chunks, [splice @archivos, 0, 30] while (@archivos); # grupos de 30 archivos.

        my %headers = ('Content-Type' => 'application/json',
                       'Authorization' => " Bearer $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_API_TOKEN'}",
                      );

        # The CloudFlare API sets a maximum of 1,200 requests in a five minute period.
        # 240 por minuto, 4 por segundo.
        if ($cloudflareZone) {
            foreach my $chunk (@chunks) {
                foreach my $sub (@SUBDOMAINS) {
                    $sub .= "." if ($sub);
                    my @files = map {$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'} . "://$sub$cloudflareDomain" . $_} @{$chunk}; # agregar la zona al principio.
                    my %data = ('files' => \@files);

                    print STDERR "[$$] cloudflare purge: sub[$sub] name[$cloudflareDomain] id[$cloudflareZone] files[" . (join(",", @files)) . "]\n";

                    my $req_url = "https://api.cloudflare.com/client/v4/zones/$cloudflareZone/purge_cache";
                    my $json = JSON::encode_json(\%data);
                    utf8::decode($json);
                    my ($resp, $http_code) = &http_delete($req_url, \%headers, $json);

                    if ($http_code ne '200') {
                        print STDERR "[$$] Error al hacer purge en el dominio [$cloudflareDomain] zona[$cloudflareZone] sub[$sub] resp[$resp]\n";
                    } else {
                        print STDERR "[$$] OK.\n";
                    }

                    Time::HiRes::sleep(0.25);
                }
            }
        }
    }

    # Actualiza fecha de modificación del semaforo en caso de que se procesen muchos archivos
    # no llegue otro proceso y lo pise.
    # El semaforo es util cuando el script se caiga o se quede pegado (sin actualizar mtime).
    utime time, time, "$DIR_SEMAF/purge_cache.lck";
}


sub check_life {
    my $t = $_[0];
    my $dif = time - $t;

    if ($dif <= $FILE_LIFE_SEGS) {
        return 1;
    }

    return 0;
}


sub http_purge {
    my $url = $_[0];

    return ('', '') if ($url eq '');
    my($request) = new HTTP::Request('PURGE', $url) || return ('', $!);
    my($response) = $UA->request($request) || return ('', $!);
    # print STDERR "[$$] status_line[" . $response->status_line . "], header: [" . $response->headers_as_string . "]\n";
    if ($response->is_success) {
        return ($response->content, $response->status_line);
    } else {
        return ('', $response->status_line);
    }
}


sub http_delete {
    my $url = $_[0];
    my $headers_ref = $_[1];
    my $data = $_[2];
    my %headers = %{$headers_ref};

    my $req = HTTP::Request->new('DELETE', $url);

    foreach my $header (keys %headers) {
        $req->header($header => $headers{$header});
    }

    $req->content($data) if ($data);

    my $response = $UA->request($req);

    return ($response->content, $response->code);
}


sub signal_callback {
    print STDERR  "Terminado por signal @_\n";
    exit(0);
}


sub delete_semaphore {
    if ($RUNNING) {
        unlink "$DIR_SEMAF/purge_cache.lck";
    }
}

==> release.12.3.1.2.tgz: cgi-cpn/prontus_purge_cache.pl <==
#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}


END {
    &delete_semaphore();
}

$SIG{INT} = \&signal_callback;
$SIG{TERM} = \&signal_callback;

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use HTTP::Request;
use JSON;
use LWP::ConnCache;
use LWP::UserAgent;
use Time::HiRes;
use glib_dbi_03;
use glib_fildir_03;
use lib_bd;
use lib_lang;
use lib_urlf5;
use lib_utils;
##/libs##

my $UA;
my $DIR_SEMAF;
my %PURGED_FILES;
my $FILE_LIFE_SEGS = 5;
my $RUNNING = 0;
my @SUBDOMAINS;

main: {
    lib_utils::setUp();
    my $prontus_id = $ARGV[0];

    if (scalar @ARGV < 1) {
        print lib_lang::get_message('global_insufficient_arguments');
        exit;
    }

    if ($prontus_id eq '' || !lib_utils::valida_prontus($prontus_id)) {
        my $msg = lib_lang::get_message('global_invalid_prontus_id')."\n";
        print $msg;
        print STDERR $msg;
        exit;
    }

    # Carga variables de configuracion de prontus.
    my $msg_load = lib_utils::load_config_by_id($prontus_id);
    if ($msg_load ne '') {
        print STDERR "$msg_load\n";
        exit;
    }

    # Conectar a BD
    my ($bd, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($bd)) {
        print STDERR $msg_err_bd, "\n";
        exit;
    }

    @SUBDOMAINS = keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_PURGE_SUBDOMAINS'}};

    # agrega elemento vacío para procesar el dominio principal
    push (@SUBDOMAINS, '');

    $DIR_SEMAF = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/semaforos";
    glib_fildir_03::check_dir($DIR_SEMAF);

    if (-f "$DIR_SEMAF/purge_cache.lck") {
        my $mtime = (stat("$DIR_SEMAF/purge_cache.lck"))[9];
        my $now = time;
        my $diff = $now - $mtime;

        if ($diff > 1800) { # 30 min.
            print STDERR lib_lang::get_interpolated_message('purge_cache_semaphore_old', $$);
            unlink "$DIR_SEMAF/purge_cache.lck";

            my $res = `ps auxww |grep 'prontus_purge_cache.pl $prontus_varglb::PRONTUS_ID' | grep -v grep | wc -l`;
            chomp($res);

            if ($res) {
                system('kill -9 `ps auxww | grep \'prontus_purge_cache.pl ' . $prontus_varglb::PRONTUS_ID . '\' | grep -v grep | awk \'{print $2}\' | grep -v ' . $$ . '`');
            }
        } else {
            exit(0); # ya existe un proceso.
        }
    }

    # escribe el semaforo y ya esta corriendo.
    glib_fildir_03::write_file("$DIR_SEMAF/purge_cache.lck", $$);
    $RUNNING = 1;

    my $espera_segs = 5;
    $UA = new LWP::UserAgent;
    $UA->timeout($espera_segs); # segs # default es 180
    $UA->agent('Prontus CMS');
    # no verifica certificados para poder acceder a sitios por ip que pueden entregar un certificado invalido
    $UA->ssl_opts(verify_hostname => 0,
              SSL_verify_mode => 0x00);
    my $conn_cache = LWP::ConnCache->new();
    $UA->conn_cache($conn_cache);

    my $dir_purgepend = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/purgepend";
    my @purgepend_chunklist = &get_purgepend_list($dir_purgepend);
    my $chunk_ref;

    while (defined($chunk_ref = shift @purgepend_chunklist)) {
        my @chunk = @{$chunk_ref};
        my %filelist;

        foreach my $file (@chunk) {
            if (-f $file) {
                print STDERR lib_lang::get_interpolated_message("purge_cache_processing_queue", $$, $file);
                %filelist = &get_filelist_hash($file, $bd, \%filelist);
            }
        }

        my $num_of_files = keys %filelist;

        print STDERR lib_lang::get_interpolated_message('purge_cache_processing_routes', $$, $num_of_files);

        if ($num_of_files) {
            &purge(\%filelist);
        }

        # Borrar archivos
        foreach my $file (@chunk) {
            unlink $file if (-f $file);
        }

        if (!scalar @purgepend_chunklist) {
            print STDERR lib_lang::get_interpolated_message('purge_cache_searching_files', $$);
            @purgepend_chunklist = &get_purgepend_list($dir_purgepend);
        }
    }

    print STDERR lib_lang::get_interpolated_message('purge_cache_end', $$);

    unlink "$DIR_SEMAF/purge_cache.lck";
}


sub get_purgepend_list {
    my $dir_purgepend = $_[0];

    my @files = glob("$dir_purgepend/*.txt");
    my @chunks;

    # Dividir array de archivos.
    push @chunks, [splice @files, 0, 50] while (@files); # grupos de 50 archivos.

    return @chunks;
}


sub get_filelist_hash {
    my $file = $_[0];
    my $bd = $_[1];
    my %hash = %{$_[2]};

    open(my $PURGEFILE, '<', "$file");

    foreach my $filetopurge (<$PURGEFILE>) {
        $filetopurge =~ s/\n//s;
        my $relpath = lib_utils::remove_front_string($filetopurge, $prontus_varglb::DIR_SERVER);
        if (!exists $hash{$relpath}) {
            $hash{$relpath} = time;
            # friendly v5 activa
            if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_ENABLE'} eq '5') {
                next if ($relpath !~ /\.(html|htm|php|asp)$/);

                if ($relpath =~ m|/site/(ta[xg])/port/[\w/\.]+_(\d+)\.\w+$|i) {
                    my $type = $1;
                    my $pag = $2;
                    my $friendly;
                    if ($type eq 'tax') {
                        $friendly = lib_urlf5::get_link_tax($bd, $relpath, $pag);
                    } else {
                        $friendly= lib_urlf5::get_link_tag($bd, $relpath, $pag);
                    }
                    $hash{$friendly} = time;
                } else {
                    # urls no tax tag, aplica a articulos y portadas
                    my $result = lib_urlf5::get_file_data($bd, $relpath);
                    if ($result) {
                        $hash{$result->{'urlf5_url'}} = time;
                    }
                }
            }
        }
    }
    close $PURGEFILE;

    return %hash;
}

sub purge {
    my %hash = %{$_[0]};

    print STDERR lib_lang::get_interpolated_message('purge_cache_active_servers', $$);

    # Agregar purge global al listado.
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_GLOBAL_PURGE'} ne '') {
        my @arr = split(/[\n\r]/, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_GLOBAL_PURGE'});
        foreach my $path (@arr) {
            next unless($path);
            next if ($path !~ /^\//); # debe empezar con /

            $hash{$path} = time if (!exists $hash{$path});
        }
    }

    # purga las rutas desde los servidores de cache
    foreach my $relpath (keys %hash) {
        if (defined $PURGED_FILES{$relpath} && &check_life($PURGED_FILES{$relpath})) {
            print STDERR lib_lang::get_interpolated_message('purge_cache_skipping', $$, $relpath);
            next;
        }

        $PURGED_FILES{$relpath} = time;

        foreach my $server (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CACHE_SERVER_NAME'}}) {
            next if (!$server);
            my $url_purge;
            if ($server =~ /^https?:\/\//) {
                $url_purge = "$server$relpath";
            } else {
                $url_purge = "https://$server$relpath";
            }

            my ($resp, $err) = &http_purge($url_purge);
            print STDERR lib_lang::get_interpolated_message('purge_cache_purge_info', $$, $server, $url_purge, $err);
        }
    }

    if (   $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE'} eq 'SI'
        && $prontus_varglb::CLOUDFLARE_API_URL ne ''
        && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_API_TOKEN'} ne ''
        && $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_ZONE'} ne ''){
        # asumimos que la zona de CLOUDFLARE_ZONES es el id alfanumerico de zona, no el dominio. Como dominio se usa el PUBLIC_SERVER_NAME.
        my $cloudflareZone = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_ZONE'};
        my $cloudflareDomain = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PUBLIC_SERVER_NAME'};

        print STDERR lib_lang::get_interpolated_message('purge_cache_cloudflare_enabled', $$, $prontus_varglb::CLOUDFLARE_API_URL);

        # Agregar purge global al listado.
        if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_GLOBAL_PURGE'} ne '') {
            my @arr = split(/[\n\r]/, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_GLOBAL_PURGE'});

            foreach my $path (@arr) {
                next unless($path);
                next if (substr($path, 0, 1) ne '/'); # debe empezar con /

                $hash{$path} = time if (!exists $hash{$path});
            }
        }

        # Se envian a la api grupos de 30 url.
        my @archivos = (keys %hash);
        my @chunks;
        push @chunks, [splice @archivos, 0, 30] while (@archivos); # grupos de 30 archivos.

        my %headers = ('Content-Type' => 'application/json',
                       'Authorization' => " Bearer $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CLOUDFLARE_API_TOKEN'}",
                      );

        # The CloudFlare API sets a maximum of 1,200 requests in a five minute period.
        # 240 por minuto, 4 por segundo.
        if ($cloudflareZone) {
            foreach my $chunk (@chunks) {
                foreach my $sub (@SUBDOMAINS) {
                    $sub .= "." if ($sub);
                    my @files = map {$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_PROTOCOLO_FRONTEND'} . "://$sub$cloudflareDomain" . $_} @{$chunk}; # agregar la zona al principio.
                    my %data = ('files' => \@files);
                    my $files = join(",", @files);
                    print STDERR lib_lang::get_interpolated_message('purge_cache_cloudflare_purge', $$, $sub, $cloudflareDomain, $cloudflareZone, $files);

                    my $req_url = "https://api.cloudflare.com/client/v4/zones/$cloudflareZone/purge_cache";
                    my $json = JSON::encode_json(\%data);
                    utf8::decode($json);
                    my ($resp, $http_code) = &http_delete($req_url, \%headers, $json);

                    if ($http_code ne '200') {
                        print STDERR lib_lang::get_interpolated_message('purge_cache_cloudflare_error', $$, $cloudflareDomain, $cloudflareZone, $sub, $resp);
                    } else {
                        print STDERR lib_lang::get_interpolated_message('purge_cache_ok', $$);
                    }

                    Time::HiRes::sleep(0.25);
                }
            }
        }

    }

    # Actualiza fecha de modificación del semaforo en caso de que se procesen muchos archivos
    # no llegue otro proceso y lo pise.
    # El semaforo es util cuando el script se caiga o se quede pegado (sin actualizar mtime).
    utime time, time, "$DIR_SEMAF/purge_cache.lck";
}

sub check_life {
    my $t = $_[0];
    my $dif = time - $t;

    if ($dif <= $FILE_LIFE_SEGS) {
        return 1;
    }

    return 0;
}

sub http_purge {
    my $url = $_[0];

    return ('', '') if ($url eq '');
    my($request) = new HTTP::Request('PURGE', $url) || return ('', $!);
    my($response) = $UA->request($request) || return ('', $!);
    # print STDERR "[$$] status_line[" . $response->status_line . "], header: [" . $response->headers_as_string . "]\n";
    if ($response->is_success) {
        return ($response->content, $response->status_line);
    } else {
        return ('', $response->status_line);
    }
}

sub http_delete {
    my $url = $_[0];
    my $headers_ref = $_[1];
    my $data = $_[2];
    my %headers = %{$headers_ref};

    my $req = HTTP::Request->new('DELETE', $url);

    foreach my $header (keys %headers) {
        $req->header($header => $headers{$header});
    }

    $req->content($data) if ($data);

    my $response = $UA->request($req);

    return ($response->content, $response->code);
}

sub signal_callback {
    print STDERR lib_lang::get_interpolated_message('global_ended_by_signal', "@_");
    exit(0);
}

sub delete_semaphore {
    if ($RUNNING) {
        unlink "$DIR_SEMAF/purge_cache.lck";
    }
}
